@@ -6,6 +6,7 @@ import json
 import string
 import atexit
 import requests
+import logging
 
 # Import Detection From Stealth
 from .stealth import stealth
@@ -111,6 +112,7 @@ class browser:
 
         await self.browser.close()
         await self.browser.close()
+        self.browser.process.communicate()
 
     async def find_redirect(self):
         try:
@@ -148,9 +150,11 @@ class browser:
             self.redirect_url = self.page.url
 
             await self.browser.close()
+            self.browser.process.communicate()
 
         except:
             await self.browser.close()
+            self.browser.process.communicate()
 
     def __format_proxy(self, proxy):
         if proxy != None:
