@@ -10,21 +10,20 @@ from selenium import webdriver
 
 
 class TikTokApi:
-    #
-    # The TikTokapi class constructor
-    #
     def __init__(self, debug=False):
+        """
+          The TikTokApi class constructor
+        """
         self.debug = debug
         if debug:
             print("Class initialized")
 
         self.userAgent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.0 Safari/537.36)"
-        # self.referrer = "https://www.tiktok.com/@ondymikula/video/6757762109670477061"
 
-    #
-    # Method that retrives data from the api
-    #
     def getData(self, api_url, b, language='en', proxy=None):
+        """
+          Method that retrives data from the api
+        """
         url = b.url + \
             "&verifyFp=" + b.verifyFp + \
             "&_signature=" + b.signature
@@ -34,17 +33,17 @@ class TikTokApi:
             'path': url.split("tiktok.com")[1],
             'scheme': 'https',
             'accept': 'application/json, text/plain, */*',
-                                       "accept-encoding": "gzip, deflate, br",
-                                       'accept-language': 'en-US,en;q=0.9',
-                                       "referrer": b.referrer,
-                                       'sec-fetch-dest': 'empty',
-                                       'sec-fetch-mode': 'cors',
-                                       'sec-fetch-site': 'same-site',
-                                       "user-agent": b.userAgent
-                                       
-                                       
-                                       
-                                       }, proxies=self.__format_proxy(proxy))
+            "accept-encoding": "gzip, deflate, br",
+            'accept-language': 'en-US,en;q=0.9',
+            "referrer": b.referrer,
+            'sec-fetch-dest': 'empty',
+            'sec-fetch-mode': 'cors',
+            'sec-fetch-site': 'same-site',
+            "user-agent": b.userAgent
+
+
+
+        }, proxies=self.__format_proxy(proxy))
         try:
             return r.json()
         except:
@@ -54,11 +53,10 @@ class TikTokApi:
 
             raise Exception('Invalid Response')
 
-    #
-    # Method that retrives data from the api
-    #
-
     def getBytes(self, api_url, b, proxy=None):
+        """
+          Method that retrives bytes from the api
+        """
         url = api_url + \
             "&_verifyFp=" + b.verifyFp + \
             "&_signature=" + b.signature
@@ -69,11 +67,10 @@ class TikTokApi:
                                        }, proxies=self.__format_proxy(proxy))
         return r.content
 
-    #
-    # Gets trending Tiktoks
-    #
-
     def trending(self, count=30, language='en', region='US', proxy=None):
+        """
+          Gets trending TikToks
+        """
         response = []
         maxCount = 99
         maxCursor = 0
@@ -105,11 +102,10 @@ class TikTokApi:
 
         return response[:count]
 
-    #
-    # Gets a specific user's tiktoks
-    #
-
     def userPosts(self, userID, secUID, count=30, language='en', region='US', proxy=None):
+        """
+          Gets a specific user's tiktoks
+        """
         response = []
         maxCount = 99
         maxCursor = 0
@@ -140,18 +136,17 @@ class TikTokApi:
 
         return response[:count]
 
-    #
-    # Gets a specific user's tiktoks by username
-    #
-
     def byUsername(self, username, count=30, proxy=None, language='en', region='US'):
+        """
+          Gets a specific user's tiktoks by username
+        """
         data = self.getUserObject(username, proxy=proxy)
         return self.userPosts(data['id'], data['secUid'], count=count, proxy=proxy, language=language, region=region)
 
-    #
-    # Gets a user's liked posts
-    #
     def userLiked(self, userID, secUID, count=30, language='en', region='US', proxy=None):
+        """
+          Gets a user's liked posts
+        """
         response = []
         maxCount = 99
         maxCursor = 0
@@ -190,21 +185,19 @@ class TikTokApi:
 
         return response[:count]
 
-    #
-    # Gets a specific user's likes by username
-    #
-
     def userLikedbyUsername(self, username, count=30, proxy=None, language='en', region='US'):
+        """
+          Gets a specific user's likes by username
+        """
         data = self.getUserObject(username, proxy=proxy)
         return self.userLiked(data['id'], data['secUid'], count=count, proxy=proxy, language=language, region=region)
 
-    #
-    # Gets tiktoks by music ID
-    #
-    # id - the sound ID
-    #
-
     def bySound(self, id, count=30, language='en', proxy=None):
+        """
+          Gets tiktoks by music ID
+
+          id - the sound ID
+        """
         response = []
         maxCount = 99
         maxCursor = 0
@@ -232,20 +225,19 @@ class TikTokApi:
 
         return response[:count]
 
-    #
-    # Gets the music object
-    #
     def getMusicObject(self, id, language='en', proxy=None):
+        """
+          Gets the music object
+        """
         api_url = "https://m.tiktok.com/api/music/detail/?musicId={}&language={}&verifyFp=".format(
             str(id), language)
         b = browser(api_url, proxy=proxy)
         return self.getData(api_url, b, proxy=proxy)
 
-    #
-    # Gets tiktoks by hashtag
-    #
-
     def byHashtag(self, hashtag, count=30, language='en', proxy=None):
+        """
+          Gets tiktoks by hashtag
+        """
         id = self.getHashtagObject(hashtag)['challengeInfo']['challenge']['id']
         response = []
         maxCount = 99
@@ -274,38 +266,37 @@ class TikTokApi:
 
         return response[:count]
 
-    #
-    # Gets tiktoks by hashtag (for use in byHashtag)
-    #
-
     def getHashtagObject(self, hashtag, language='en', proxy=None):
+        """
+          Gets tiktoks by hashtag (for use in byHashtag)
+        """
         api_url = "https://m.tiktok.com/api/challenge/detail/?challengeName={}&language={}".format(
             str(hashtag.encode('utf-8'))[2:-1].replace("\\x", "%").upper(), language)
         b = browser(api_url, proxy=proxy)
         return self.getData(api_url, b, proxy=proxy)
 
-    #
-    # Get a tiktok object with another tiktok's id
-    #
     def getRecommendedTikToksByVideoID(self, id, language='en', proxy=None):
+        """
+           Get a tiktok object with another tiktok's id
+        """
         api_url = "https://m.tiktok.com/share/item/list?secUid=&id={}&type=0&count=24&minCursor=0&maxCursor=0&shareUid=&recType=3&lang={}&verifyFp=".format(
             id, language)
         b = browser(api_url, proxy=proxy)
         return self.getData(api_url, b, proxy=proxy)['body']
 
-    #
-    # Gets a tiktok object by ID
-    #
     def getTikTokById(self, id, language='en', proxy=None):
+        """
+          Gets a tiktok object by ID
+        """
         api_url = "https://m.tiktok.com/api/item/detail/?itemId={}&language={}&verifyFp=".format(
             id, language)
         b = browser(api_url, proxy=proxy)
         return self.getData(api_url, b, proxy=proxy)
 
-    #
-    # Get a tiktok object by url
-    #
     def getTikTokByUrl(self, url, language='en', proxy=None):
+        """
+          Get a tiktok object by url
+        """
         if "@" in url and "/video/" in url:
             post_id = url.split("/video/")[1].split("?")[0]
         else:
@@ -314,43 +305,41 @@ class TikTokApi:
 
         return self.getTikTokById(post_id, language=language, proxy=proxy)
 
-    #
-    # Discover page, consists challenges (hashtags)
-    #
-
     def discoverHashtags(self, proxy=None):
+        """
+          Discover page, consists challenges (hashtags)
+        """
         api_url = "https://m.tiktok.com/node/share/discover?noUser=1&userCount=30&scene=0&verifyFp="
         b = browser(api_url, proxy=proxy)
         return self.getData(api_url, b, proxy=proxy)['body'][1]['exploreList']
 
-    #
-    # Discover page, consists of music
-    #
-
     def discoverMusic(self, proxy=None):
+        """
+          Discover page, consists of music
+        """
         api_url = "https://m.tiktok.com/node/share/discover?noUser=1&userCount=30&scene=0&verifyFp="
         b = browser(api_url, proxy=proxy)
         return self.getData(api_url, b, proxy=proxy)['body'][2]['exploreList']
-    #
-    # Gets a user object for id and secUid
-    #
 
     def getUserObject(self, username, language='en', proxy=None):
+        """
+          Gets a user object for id and secUid
+        """
         return self.getUser(username, language, proxy=proxy)['userInfo']['user']
 
-    #
-    # Gets the full exposed user object
-    #
     def getUser(self, username, language='en', proxy=None):
+        """
+          Gets the full exposed user object
+        """
         api_url = "https://m.tiktok.com/api/user/detail/?uniqueId={}&language={}".format(
             username, language)
         b = browser(api_url, proxy=proxy)
         return self.getData(api_url, b, proxy=proxy)
 
-    #
-    # Get Suggested Users for given userID
-    #
     def getSuggestedUsersbyID(self, count=30, userId='6745191554350760966', language='en', proxy=None):
+        """
+          Get Suggested Users for given userID
+        """
         api_url = "https://m.tiktok.com/node/share/discover?noUser=0&pageId={}&userId={}&userCount={}&scene=15&verifyFp=".format(
             userId, userId, str(count))
         b = browser(api_url, proxy=proxy)
@@ -360,10 +349,10 @@ class TikTokApi:
             res.append(x['cardItem'])
         return res[:count]
 
-    #
-    # Crawler for Suggested Users
-    #
     def getSuggestedUsersbyIDCrawler(self, count=30, startingId='6745191554350760966', language='en', proxy=None):
+        """
+          Crawler for Suggested Users
+        """
         users = []
         unusedIDS = [startingId]
         while len(users) < count:
@@ -379,10 +368,10 @@ class TikTokApi:
 
         return users[:count]
 
-    #
-    # Get suggested hashtags given userID
-    #
     def getSuggestedHashtagsbyID(self, count=30, userId='6745191554350760966', language='en', proxy=None):
+        """
+          Get suggested hashtags given userID
+        """
         api_url = "https://m.tiktok.com/node/share/discover?noUser=0&pageId={}&userId={}&userCount={}&scene=15&verifyFp=".format(
             userId, userId, str(count))
         b = browser(api_url, proxy=proxy)
@@ -392,10 +381,10 @@ class TikTokApi:
             res.append(x['cardItem'])
         return res[:count]
 
-    #
-    # Crawler for Suggested Hashtags
-    #
     def getSuggestedHashtagsbyIDCrawler(self, count=30, startingId='6745191554350760966', language='en', proxy=None):
+        """
+          Crawler for Suggested Hashtags
+        """
         hashtags = []
         ids = self.getSuggestedUsersbyIDCrawler(
             count=count, startingId=startingId, language=language, proxy=proxy)
@@ -411,10 +400,10 @@ class TikTokApi:
 
         return hashtags[:count]
 
-    #
-    # Get suggested music by given userID
-    #
     def getSuggestedMusicbyID(self, count=30, userId='6745191554350760966', language='en', proxy=None):
+        """
+          Get suggested music by given userID
+        """
         api_url = "https://m.tiktok.com/node/share/discover?noUser=0&pageId={}&userId={}&userCount={}&scene=15&verifyFp=".format(
             userId, userId, str(count))
         b = browser(api_url, proxy=proxy)
@@ -424,10 +413,10 @@ class TikTokApi:
             res.append(x['cardItem'])
         return res[:count]
 
-    #
-    # Crawler for Suggested Music
-    #
     def getSuggestedMusicIDCrawler(self, count=30, startingId='6745191554350760966', language='en', proxy=None):
+        """
+          Crawler for Suggested Music
+        """
         musics = []
         ids = self.getSuggestedUsersbyIDCrawler(
             count=count, startingId=startingId, language=language, proxy=proxy)
@@ -443,52 +432,34 @@ class TikTokApi:
 
         return musics[:count]
 
-    #
-    # Downloads video from TikTok using a TikTok object
-    #
-
     def get_Video_By_TikTok(self, data, proxy=None):
+        """
+          Downloads video from TikTok using a TikTok object
+        """
         api_url = data['video']['downloadAddr']
         return self.get_Video_By_DownloadURL(api_url, proxy=proxy)
 
-    #
-    # Downloads video from TikTok using download url in a tiktok object
-    #
     def get_Video_By_DownloadURL(self, download_url, proxy=None):
+        """
+          Downloads video from TikTok using download url in a tiktok object
+        """
         b = browser(download_url, proxy=proxy)
         return self.getBytes(download_url, b, proxy=proxy)
 
-    #
-    # Gets the source url of a given url for a tiktok
-    #
-    # video_url - the url of the video
-    # return_bytes - 0 is just the url, 1 is the actual video bytes
-    # chromedriver_path - path to your chrome driver executible
-    #
-
     def get_Video_By_Url(self, video_url, return_bytes=0, chromedriver_path=None):
+        """
+           Gets the source url of a given url for a tiktok
+
+            video_url - the url of the video
+            return_bytes - 0 is just the url, 1 is the actual video bytes
+            chromedriver_path - path to your chrome driver executable 
+        """
         raise Exception("Deprecated. Other Methods Work Better.")
-        if chromedriver_path != None:
-            driver = webdriver.Chrome(executable_path=chromedriver_path)
-        else:
-            driver = webdriver.Chrome()
-        driver.get(video_url)
-        time.sleep(5)
 
-        soup = BeautifulSoup(driver.page_source, 'html.parser')
-        data = json.loads(soup.find_all(
-            'script', attrs={"id": "videoObject"})[0].text)
-
-        if return_bytes == 0:
-            return data['contentUrl']
-        else:
-            r = requests.get(data['contentUrl'])
-            return r.content
-
-    #
-    # No Water Mark
-    #
     def get_Video_No_Watermark_ID(self, video_id, return_bytes=0, proxy=None):
+        """
+          No Water Mark
+        """
         video_info = self.getTikTokById(video_id)
         video_url = video_info["itemInfo"]["itemStruct"]["video"]["downloadAddr"]
         headers = {"User-Agent": "okhttp", "Range": "bytes=1000-80000"}
@@ -503,23 +474,70 @@ class TikTokApi:
             return video_url_no_wm
         else:
             headers = {"User-Agent": "okhttp"}
-            video_data_no_wm = requests.get(video_url_no_wm, params=None, headers=headers)
+            video_data_no_wm = requests.get(
+                video_url_no_wm, params=None, headers=headers)
             return video_data_no_wm.content
-            
 
-
-    def get_Video_No_Watermark(self, video_url, return_bytes=0, proxy=None):
+    def get_Video_No_Watermark_Faster(self, video_url, return_bytes=0, proxy=None):
+        """
+          No Water Mark method, but may be faster
+        """
         video_id = video_url.split("/video/")[1].split("?")[0]
         return self.get_Video_No_Watermark_ID(video_id, return_bytes, proxy=proxy)
 
+    def get_Video_No_Watermark(self, video_url, return_bytes=0, proxy=None):
+        """
+          Gets the video with no watermark that you can open in any browser w/ any useragent
+        """
+        r = requests.get(video_url, headers={"method": "GET",
+                                             "accept-encoding": "utf-8",
+                                             "user-agent": self.userAgent
+                                             }, proxies=self.__format_proxy(proxy))
+
+        data = r.text
+
+        check = data.split('video":{"urls":["')
+        if len(check) > 1:
+            contentURL = check[1].split("\"")[0]
+            r = requests.get(contentURL, headers={"method": "GET",
+                                                  "accept-encoding": "gzip, deflate, br",
+                                                  'accept-Language': 'en-US,en;q=0.9',
+                                                  'Range': 'bytes=0-200000',
+                                                  'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
+                                                  "user-agent": self.userAgent,
+                                                  }, proxies=self.__format_proxy(proxy))
+
+            tmp = r.text.split("vid:")
+            if len(tmp) > 1:
+                key = tmp[1].split("%")[0]
+
+                if key[-1:] == ' ':
+                    key = key[1:]
+
+                if key[:1] == ' ':
+                    key = key[:-1]
+
+            else:
+                key = ""
+
+            cleanVideo = "https://api.tiktokv.com/aweme/v1/playwm/?video_id=" + key
+
+            b = browser(cleanVideo, find_redirect=True, proxy=proxy)
+
+            if return_bytes == 0:
+                return b.redirect_url
+            else:
+                r = requests.get(
+                    b.redirect_url, proxies=self.__format_proxy(proxy))
+                return r.content
     #
     # PRIVATE METHODS
     #
 
-    #
-    # Formats the proxy object
-    #
     def __format_proxy(self, proxy):
+        """
+          Formats the proxy object
+        """
         if proxy != None:
             return {
                 'http': proxy,
